#include "gui_label.h"

#include "imgui.h"

/*
    The GUI library allows for formatted texts to be used. For example: ImGui::Text("Int value: %d", int_variable);
    %d will always get filled with current int_variable values. However this is impossible to replicate with this class
    due to inability to save va_list (argumet list for formatted text), also the values would not get updated. 
    It could be theorethically possible to use this class as observer for single value (through pointer), but
    that would complicate this class for no reason.

    For formatted text in GUI use gui_direct (calls user defined function, that calls GUI directly)
*/

gui_label::gui_label(const char *label, int x, int y, int width, app_workspace_ns::font_size font_size) {
    this->label = label;
    this->x = x;
    this->y = y;
    this->width = width;
    this->font_size = font_size;
}

gui_label::~gui_label() {
}

void gui_label::render_element() {
    gui_element::render_element();

    app_workspace_ns::font_size current_fs = app_workspace::get_instance()->get_font_size();

    // if element is requested with different font size
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(font_size);

    if (width > -1) {
        // 480 = window width, 10 = padding
        float wrap_width = (width + (x > -1 ? x : 0)) > 480 ? (480 - x - 10) : width;
        ImVec2 cpos = ImGui::GetCursorPos();
        ImGui::PushTextWrapPos(cpos.x + wrap_width);
    }

    ImGui::TextWrapped(this->label);
    //ImGui::Text(this->label);

    if (width > -1) {
        ImGui::PopTextWrapPos();
    }

    // return back original font size (otherwise all subsequent elements would use font_size, unless specified)
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(current_fs);
}

void gui_label::set_label(const char* label) {
    this->label = label;
}