#include "gui_observer.h"

#include <spdlog/spdlog.h>

// IMPORTANT - for string only char* is supplied into target, not char**
gui_observer::gui_observer(observed_type type, void* target, int x, int y, int width,
                           app_workspace_ns::font_size font_size)
{
    this->type = type;
    this->target = target;
    this->x = x;
    this->y = y;
    this->width = width; // width might not make sense with this 
    this->font_size = font_size;
}

void gui_observer::render_element() {
    gui_element::render_element();

    app_workspace_ns::font_size current_fs = app_workspace::get_instance()->get_font_size();

    // if element is requested with different font size
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(font_size);

    if (width > -1) {
        // 480 = window width, 10 = padding - TODO constants for those values
        float wrap_width = (width + (x > -1 ? x : 0)) > 480 ? (480 - x - 10) : width;
        ImVec2 cpos = ImGui::GetCursorPos();
        ImGui::PushTextWrapPos(cpos.x + wrap_width);
    }

    switch (type) {
        case OBS_INT:
            ImGui::TextWrapped("%d", *((int*) target));
            break;
        case OBS_LONG:
            ImGui::TextWrapped("%ld", *((long*) target));
            break;
        case OBS_FLOAT:
            ImGui::TextWrapped("%.3f", *((float*) target));
            break;
        case OBS_STRING:

            ImGui::TextWrapped("%s", (const char*) target);
            /*
            char* tmp = *((char**) target);
            // true if const char* is supplied, false if char[] is supplied
            if (tmp) 
                ImGui::TextWrapped("%s", tmp);
            else 
                ImGui::TextWrapped("%s", (char*) target);
            */
            break;
        default:
            spdlog::error("gui_observer.cpp - Unknown observed type. This can only occur on memory corruption (overflow)");
    }

    if (width > -1)
        ImGui::PopTextWrapPos();

    // return back original font size (otherwise all subsequent elements would use font_size, unless specified)
    if (current_fs != font_size)
        app_workspace::get_instance()->use_font_size(current_fs);
}