---
--- creating user and database
---
CREATE USER 'pi'@'localhost' IDENTIFIED BY 'raspberry';
CREATE DATABASE control_system DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_unicode_ci;
GRANT ALL PRIVILEGES ON control_system.* to 'pi'@'localhost' IDENTIFIED BY 'raspberry';
USE control_system;
---
--- creating tables
---
CREATE TABLE IF NOT EXISTS cs_users (
    id INT(8) AUTO_INCREMENT PRIMARY KEY,
    status INT(1) NOT NULL DEFAULT 1,
    role INT(1) NOT NULL,
    username VARCHAR(64) NOT NULL UNIQUE,
    password BINARY(64) NOT NULL,
    rfid_serial TINYBLOB NULL,
    description VARCHAR(2048),
    name VARCHAR(128) NOT NULL,
    lastname VARCHAR(64) NOT NULL,
    measure_count INT(8) NOT NULL DEFAULT 0,
    reg_date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    last_login_date TIMESTAMP,
    date_of_birth DATE NOT NULL
);
---
CREATE TABLE IF NOT EXISTS cs_measurements (
    id INT(8) AUTO_INCREMENT PRIMARY KEY,
    status INT(1) NOT NULL DEFAULT 1,
    measurement_number INT(8) NOT NULL,
    measurement_part INT(8) NOT NULL DEFAULT 1,
    measuree_id INT(8) NOT NULL,
    measurer_id INT(8) NOT NULL,
    measuring_length INT(8),
    measuring_start TIMESTAMP NOT NULL,
    measuring_end   TIMESTAMP NOT NULL,
    description     VARCHAR(2048),
    value_length    INT(1) DEFAULT 8,
    value_count     INT(8),
    m_values        MEDIUMBLOB,
    x_val_median    DOUBLE NOT NULL,
    x_val_avg       DOUBLE NOT NULL,
    all_val_median  DOUBLE NOT NULL,
    all_val_avg     DOUBLE NOT NULL,
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    CONSTRAINT FK_measuree_user FOREIGN KEY (measuree_id) REFERENCES cs_users(id),
    CONSTRAINT FK_measuring_user FOREIGN KEY (measurer_id) REFERENCES cs_users(id)
);
---
--- inserting test data
---
-- creates 3 test users, 1 pacient, 1 employee, 1 admin
--    blue tag: 527239190 | white card: 19211526
-- can't input test rfid tags
INSERT INTO cs_users(role, username, password, description, name, lastname, date_of_birth) 
    VALUES (0, "p", "9999", "This user was created to test pacient", "Pacient 1", "Test 1", "2000-2-20");
INSERT INTO cs_users(role, username, password, description, name, lastname, date_of_birth) 
    VALUES (1, "e", "9999", "This user was created to test employee", "Employee 1", "Neco 1", "2000-2-20");
INSERT INTO cs_users(role, username, password, description, name, lastname, date_of_birth) 
    VALUES (2, "a", "9999", "This user was created to test admin", "Admin 1", "Pokus 1", "2000-2-20");
--
-- creates measurements measurement number should be unique for every user (every user has measurings 1, 2..)
-- all created test measurements are for user p (pacient)
INSERT INTO cs_measurements(measurement_number, measurement_part, measuree_id, measurer_id, measuring_length, 
        measuring_start, measuring_end, description, value_length, value_count, m_values, 
        x_val_median, x_val_avg, all_val_median, all_val_avg)
    VALUES (1, 1, 1, 1, 16574000, "2022-01-01 12:00:00", "2022-01-01 12:00:17", "test measuring 1", 8, 50,
            "this is binary values", 7.2, 7.1, 7.5, 7.4);
INSERT INTO cs_measurements(measurement_number, measurement_part, measuree_id, measurer_id, measuring_length, 
        measuring_start, measuring_end, description, value_length, value_count, m_values, 
        x_val_median, x_val_avg, all_val_median, all_val_avg)
    VALUES (2, 1, 1, 1, 90000000, "2022-01-01 12:00:00", "2022-01-01 12:01:30", "test measuring 2", 8, 5000,
            "this is binary values", 1234.0, 1200.3, 1180.7, 1250.4);
INSERT INTO cs_measurements(measurement_number, measurement_part, measuree_id, measurer_id, measuring_length, 
        measuring_start, measuring_end, description, value_length, value_count, m_values, 
        x_val_median, x_val_avg, all_val_median, all_val_avg)
    VALUES (3, 1, 2, 1, 60000000, "2022-01-01 12:00:00", "2022-01-01 12:01:00", "test measuring 3", 8, 2500,
            "this is binary values", 213.1, 142.5, 166.8, 123.8);
-- addint more following record simulate measuring number 2 as continuous
INSERT INTO cs_measurements(measurement_number, measurement_part, measuree_id, measurer_id, measuring_length, 
        measuring_start, measuring_end, description, value_length, value_count, m_values, 
        x_val_median, x_val_avg, all_val_median, all_val_avg)
    VALUES (2, 2, 1, 1, 90000000, "2022-01-01 12:02:00", "2022-01-01 12:02:40", "test measuring 2", 8, 5000,
            "this is binary values", 124.0, 120.3, 180.7, 125.4);
INSERT INTO cs_measurements(measurement_number, measurement_part, measuree_id, measurer_id, measuring_length, 
        measuring_start, measuring_end, description, value_length, value_count, m_values, 
        x_val_median, x_val_avg, all_val_median, all_val_avg)
    VALUES (2, 3, 1, 1, 90000000, "2022-01-01 12:03:00", "2022-01-01 12:05:50", "test measuring 2", 8, 5000,
            "this is binary values", 6234.0, 4200.3, 5180.7, 1250.4);